--[[

Farrons Enhancer Module v2.3
License: Beerware

]]--


do
	local _, playerClass = UnitClass("player")
	if playerClass ~= "SHAMAN" then return end
end

local Faceroller = _G["Faceroller"]

local opt
local b_mw
local b_ls
local md_fs
local md_ss

local skills = {
	[17364] = 0,	-- Stormstrike
	[49233] = 0,	-- Flame Shock
	[8042] = 0,		-- Earth Shock
	[60103] = 0,	-- Lava Lash
	[324] = 0,		-- Lightning Shield
	[61657] = 0,	-- Fire Nova
	[30823] = 0,	-- Shamanistic Rage
	[403] = 0,		-- Lightning Bolt for Maelstrom Weapon
	[51533] = 0,	-- Feral Spirit
	[2894] = 0,		-- Fire Elemental Totem
	[58734] = 0,	-- Magma Totem
}

local Init = function(opts)

	Faceroller:RegisterBuff(51532)
	Faceroller:RegisterBuff(324)
	Faceroller:RegisterDebuff(49233, true)
	Faceroller:RegisterDebuff(17364, true)
	
	b_mw = Faceroller.gsi[51532]
	b_ls = Faceroller.gsi[324]
	md_fs = Faceroller.gsi[49233]
	md_ss = Faceroller.gsi [17364]

    if not opts.rotation then
        opts.rotation = 1
        opts.use_shamanistic_rage_low = false
		opts.use_shamanistic_rage_high = false
		opts.ls_refresh = 5
		opts.use_magma_totem_high = true
		opts.use_magma_totem_low = true
		opts.fs_refresh = 3
		
    end

    opts.gui = {
	
		spell_desc_1 = {
            type = "description",
            order = 1,
            name = "Rotation:",
        },
       
        rotation_chooser = {
            type = "range",
            order = 2,
            width = "full",
            name = function ()
                if opts.rotation == 1 then rotaName = "1. Simcraft" end
                if opts.rotation == 2 then rotaName = "2. EJ: BiS gear and set-up discussion" end
                return rotaName
            end,
            min = 1,
            max = 2,
            step = 1,
            get = function ()
                return opts.rotation
            end,
            set = function (i, v)
                opts.rotation = v
				if DEBUG then print("Using Rotation: " .. v) end
            end,
        },
		
        rotation_desc_2 = {
            type = "description",
            order = 3,
            name = "Shamanistic Rage:",
        },
		
		shamanistic_rage_chooser_1 = {
			type = "toggle",
			order = 4,
			name = "On Cooldown(2pt10)",
			get = function ()
				return opts.use_shamanistic_rage_high
			end,
			set = function (i, v)
				opts.use_shamanistic_rage_high = v
			end,
        },
			
		shamanistic_rage_chooser_2 = {
			type = "toggle",
			order = 5,
			name = "Only at low Mana",
			get = function ()
				return opts.use_shamanistic_rage_low
			end,
			set = function (i, v)
				opts.use_shamanistic_rage_low = v
			end,
        },
		
		spell_desc_3 = {
            type = "description",
            order = 7,
            name = "Magma Totem",
        },
		
		magma_totem_chooser_1 = {
			type = "toggle",
			order = 8,
			name = "Use Magma Totem",
			get = function ()
				return opts.use_magma_totem_high
			end,
			set = function (i, v)
				opts.use_magma_totem_high = v
			end,
        },
		
		Magma_Totem_chooser_2 = {
			type = "toggle",
			order = 9,
			name = "Low Prio refresh",
			get = function ()
				return opts.use_magma_totem_low
			end,
			set = function (i, v)
				opts.use_magma_totem_low = v
			end,
        },
		

		spell_desc_4 = {
            type = "description",
            order = 10,
            name = "Low Priority Lightning Shied (useless if you choose the Simcraft rotation):",
        },

        ls_refresh = {
            type = "range",
            order = 11,
            width = "full",
            name = function ()
                if opts.ls_refresh == 0 then lsName = "Only when buff not active" end
                if opts.ls_refresh == 1 then lsName = "Refresh Lightning Shied at 1 Stack" end
                if opts.ls_refresh == 2 then lsName = "Refresh Lightning Shied at 2 Stacks" end
                if opts.ls_refresh == 3 then lsName = "Refresh Lightning Shied at 3 Stacks" end
                if opts.ls_refresh == 4 then lsName = "Refresh Lightning Shied at 4 Stacks" end
                if opts.ls_refresh == 5 then lsName = "Refresh Lightning Shied at 5 Stacks" end
                if opts.ls_refresh == 6 then lsName = "Refresh Lightning Shied at 6 Stacks" end
                if opts.ls_refresh == 7 then lsName = "Refresh Lightning Shied at 7 Stacks" end
                if opts.ls_refresh == 8 then lsName = "Refresh Lightning Shied at 8 Stacks" end
                return lsName
            end,
            min = 0,
            max = 8,
            step = 1,
            get = function ()
                return opts.ls_refresh
            end,
            set = function (i, v)
                opts.ls_refresh = v
				if DEBUG then print("LS refresh at n Stacks: " .. v) end
            end,
			},
			
		spell_desc_5 = {
            type = "description",
            order = 12,
            name = "Don't suggests Earth Shock when Flame Shock duration is",
        },
		
		fs_refresh = {
            type = "range",
            order = 13,
            width = "full",
            name = function ()
				if opts.fs_refresh == 0 then lsName = "No ES skipping" end
				if opts.fs_refresh == 1 then lsName = "=< 0.25" end
				if opts.fs_refresh == 2 then lsName = "=< 0.50" end
				if opts.fs_refresh == 3 then lsName = "=< 0.75" end
				if opts.fs_refresh == 4 then lsName = "=< 1.00" end
				if opts.fs_refresh == 5 then lsName = "=< 1.25" end
				if opts.fs_refresh == 6 then lsName = "=< 1.50" end
				if opts.fs_refresh == 7 then lsName = "=< 1.75" end
				if opts.fs_refresh == 8 then lsName = "=< 2.00" end
				if opts.fs_refresh == 9 then lsName = "=< 2.25" end
				if opts.fs_refresh == 10 then lsName = "=< 2.50" end
				if opts.fs_refresh == 11 then lsName = "=< 2.75" end
				if opts.fs_refresh == 12 then lsName = "=< 3.00" end
				if opts.fs_refresh == 13 then lsName = "=< 3.25" end
				if opts.fs_refresh == 14 then lsName = "=< 3.50" end
				if opts.fs_refresh == 15 then lsName = "=< 3.75" end
				if opts.fs_refresh == 16 then lsName = "=< 4.00" end
				if opts.fs_refresh == 17 then lsName = "=< 4.25" end
				if opts.fs_refresh == 18 then lsName = "=< 4.50" end
				if opts.fs_refresh == 19 then lsName = "=< 4.75" end
				if opts.fs_refresh == 20 then lsName = "=< 5.00" end
				if opts.fs_refresh == 21 then lsName = "=< 5.25" end
				if opts.fs_refresh == 22 then lsName = "=< 5.50" end
				if opts.fs_refresh == 23 then lsName = "=< 5.75" end
                return lsName
            end,
            min = 0,
            max = 23,
            step = 1,
            get = function ()
                return opts.fs_refresh
            end,
            set = function (i, v)
                opts.fs_refresh = v

            end,
			},
			
		sr_use = {
			type = "range",
			order = 6,
			width = "full",
			name = function ()
				if opts.sr_use == 1 then lsName = "Use SR at <1500 Mana" end
				if opts.sr_use == 2 then lsName = "Use SR at <2000 Mana" end
				if opts.sr_use == 3 then lsName = "Use SR at <2500 Mana" end
				if opts.sr_use == 4 then lsName = "Use SR at <3000 Mana" end
				if opts.sr_use == 5 then lsName = "Use SR at <3500 Mana" end
				if opts.sr_use == 6 then lsName = "Use SR at <4000 Mana" end
				if opts.sr_use == 7 then lsName = "Use SR at <4500 Mana" end
				if opts.sr_use == 8 then lsName = "Use SR at <5000 Mana" end
				if opts.sr_use == 9 then lsName = "Use SR independently from your Mana" end
				return lsName
			end,
			min = 1,
			max = 9,
			step = 1,
				get = function ()
			return opts.sr_use
			end,
			set = function (i, v)
				opts.sr_use = v
			end,
		},
	}
				
    opt = opts
	
	ci_tl_c = 0
	ci_tr_c = 0
	ci_bl_c = 0
	ci_br_c = 0

	
	return skills
end

local NextHit = function(gcd, spells, buffs, debuffs, myDebuffs)

--FS
	local cd = spells[51533].cd
	if cd < 2 then
		if ci_tl_c ~= 1 then
			Faceroller:CornerIndicatorSetColor ("TOPRIGHT", 0, 1, 0, 1)
			ci_tl_c = 1
		end
	elseif cd < 10 then
		if ci_tl_c ~= 3 then
			Faceroller:CornerIndicatorSetColor ("TOPRIGHT", 1, 1, 0, 1)
			ci_tl_c = 3
		end
	else
		if ci_tl_c ~= 2 then
			Faceroller:CornerIndicatorSetColor ("TOPRIGHT", 1, 0, 0, 1)
			ci_tl_c = 2
		end
	end
	
	--SR

	local cd = spells[30823].cd
	if cd < 2 then
		if ci_tr_c ~= 1 then
			Faceroller:CornerIndicatorSetColor ("TOPLEFT", 0, 1, 0, 1)
			ci_tr_c = 1
		end
	elseif cd < 10 then
		if ci_tr_c ~= 3 then
			Faceroller:CornerIndicatorSetColor ("TOPLEFT", 1, 1, 0, 1)
			ci_tr_c = 3
		end
	else
		if ci_tr_c ~= 2 then
			Faceroller:CornerIndicatorSetColor ("TOPLEFT", 1, 0, 0, 1)
			ci_tr_c = 2
		end
	end

	
	--FET
	
	local cd = spells[2894].cd
	if cd < 2 then
		if ci_bl_c ~= 1 then
			Faceroller:CornerIndicatorSetColor ("BOTTOMLEFT", 0, 1, 0, 1)
			ci_bl_c = 1
		end
	elseif cd < 10 then
		if ci_bl_c ~= 3 then
			Faceroller:CornerIndicatorSetColor ("BOTTOMLEFT", 1, 1, 0, 1)
			ci_bl_c = 3
		end
	else
		if ci_bl_c ~= 2 then
			Faceroller:CornerIndicatorSetColor ("BOTTOMLEFT", 1, 0, 0, 1)
			ci_bl_c = 2
		end
	end
	
	
	if opt.rotation == 1 then
		--Simcraft
		local th = max (0.5, gcd)
		th = th + 0.1
		local n = 0
		
		if opt.use_shamanistic_rage_low then
			if 
			spells[30823].cd < th and (UnitPower("player") < 1500) and opt.sr_use == 1 or
			spells[30823].cd < th and (UnitPower("player") < 2000) and opt.sr_use == 2 or
			spells[30823].cd < th and (UnitPower("player") < 2500) and opt.sr_use == 3 or
			spells[30823].cd < th and (UnitPower("player") < 3000) and opt.sr_use == 4 or
			spells[30823].cd < th and (UnitPower("player") < 3500) and opt.sr_use == 5 or
			spells[30823].cd < th and (UnitPower("player") < 4000) and opt.sr_use == 6 or
			spells[30823].cd < th and (UnitPower("player") < 4500) and opt.sr_use == 7 or
			spells[30823].cd < th and (UnitPower("player") < 5000) and opt.sr_use == 8 or
			spells[30823].cd < th and opt.sr_use == 9 then
				n = 30823
			end
		end
		
		-- MT refresh
		if opt.use_magma_totem_low then
			local _, totemname, start, duration = GetTotemInfo(1)
				if not totemname or totemname == "" then
				n = 58734
			end
   
			if start then
				local timeleft = start + duration - GetTime()	
					if timeleft < 6 then
						n = 58734
					end
			end
		end
		-- /MT refresh
		
		if spells[60103].cd < th then
			n = 60103
		end
				
		b = buffs[b_ls]
		if b.active == false then
			n = 324
		end

		if spells[61657].cd < th then
			n = 61657
		end
		
		-- MT
		
		if opt.use_magma_totem_high then
			local _, totemname, start, duration = GetTotemInfo(1)
				if not totemname or totemname == "" then
				n = 58734
			end
   
			if start then
				local timeleft = start + duration - GetTime()	
					if timeleft < 2 then
						n = 58734
					end
			end
		end
		
		-- /MT

		--es skipping
		b = myDebuffs[md_fs]
			if 
			spells[8042].cd < th and opt.fs_refresh == 0 or
			spells[8042].cd < th and b.time_left > 0.25 and opt.fs_refresh == 1 or
			spells[8042].cd < th and b.time_left > 0.50 and opt.fs_refresh == 2 or
			spells[8042].cd < th and b.time_left > 0.75 and opt.fs_refresh == 3 or
			spells[8042].cd < th and b.time_left > 1.00 and opt.fs_refresh == 4 or
			spells[8042].cd < th and b.time_left > 1.25 and opt.fs_refresh == 5 or
			spells[8042].cd < th and b.time_left > 1.50 and opt.fs_refresh == 6 or
			spells[8042].cd < th and b.time_left > 1.75 and opt.fs_refresh == 7 or
			spells[8042].cd < th and b.time_left > 2.00 and opt.fs_refresh == 8 or
			spells[8042].cd < th and b.time_left > 2.25 and opt.fs_refresh == 9 or
			spells[8042].cd < th and b.time_left > 2.50 and opt.fs_refresh == 10 or
			spells[8042].cd < th and b.time_left > 2.75 and opt.fs_refresh == 11 or
			spells[8042].cd < th and b.time_left > 3.00 and opt.fs_refresh == 12 or
			spells[8042].cd < th and b.time_left > 3.25 and opt.fs_refresh == 13 or
			spells[8042].cd < th and b.time_left > 3.50 and opt.fs_refresh == 14 or
			spells[8042].cd < th and b.time_left > 3.75 and opt.fs_refresh == 15 or
			spells[8042].cd < th and b.time_left > 4.00 and opt.fs_refresh == 16 or
			spells[8042].cd < th and b.time_left > 4.25 and opt.fs_refresh == 17 or
			spells[8042].cd < th and b.time_left > 4.50 and opt.fs_refresh == 18 or
			spells[8042].cd < th and b.time_left > 4.75 and opt.fs_refresh == 19 or
			spells[8042].cd < th and b.time_left > 5.00 and opt.fs_refresh == 20 or
			spells[8042].cd < th and b.time_left > 5.25 and opt.fs_refresh == 21 or
			spells[8042].cd < th and b.time_left > 5.50 and opt.fs_refresh == 22 or
			spells[8042].cd < th and b.time_left > 5.75 and opt.fs_refresh == 23 then
				n = 8042
			end	
		-- /es skipping
		
		b = myDebuffs[md_fs]
		if b.active == false and spells[49233].cd < th then
			n = 49233
		end


		if spells[17364].cd < th then
			n = 17364
		end

				
		if opt.use_shamanistic_rage_low then
			if 
				spells[30823].cd < th and (UnitPower("player") < 800) and opt.sr_use == 1 or
				spells[30823].cd < th and (UnitPower("player") < 1000) and opt.sr_use == 2 or
				spells[30823].cd < th and (UnitPower("player") < 1200) and opt.sr_use == 3 or
				spells[30823].cd < th and (UnitPower("player") < 1400) and opt.sr_use == 4 or
				spells[30823].cd < th and (UnitPower("player") < 1600) and opt.sr_use == 5 or
				spells[30823].cd < th and (UnitPower("player") < 1800) and opt.sr_use == 6 or
				spells[30823].cd < th and (UnitPower("player") < 2000) and opt.sr_use == 7 or
				spells[30823].cd < th and (UnitPower("player") < 2200) and opt.sr_use == 8 then
					n = 30823
			end
		end
		
		
		if opt.use_shamanistic_rage_high then
			if spells[30823].cd < th then
				n = 30823
			end
		end
		
		b = buffs[b_mw]
		if b.count == 5 then
			n = 403
		end

		return n
	

	elseif opt.rotation == 2 then
		--EJ
		local th = max (0.5, gcd)
		th = th + 0.1
		local n = 0

		if opt.use_shamanistic_rage_low then
			if 
				spells[30823].cd < th and (UnitPower("player") < 1500) and opt.sr_use == 1 or
				spells[30823].cd < th and (UnitPower("player") < 2000) and opt.sr_use == 2 or
				spells[30823].cd < th and (UnitPower("player") < 2500) and opt.sr_use == 3 or
				spells[30823].cd < th and (UnitPower("player") < 3000) and opt.sr_use == 4 or
				spells[30823].cd < th and (UnitPower("player") < 3500) and opt.sr_use == 5 or
				spells[30823].cd < th and (UnitPower("player") < 4000) and opt.sr_use == 6 or
				spells[30823].cd < th and (UnitPower("player") < 4500) and opt.sr_use == 7 or
				spells[30823].cd < th and (UnitPower("player") < 5000) and opt.sr_use == 8 or
				spells[30823].cd < th and opt.sr_use == 9 then
					n = 30823
			end
		end
		
		b = buffs[b_ls]
			if 	b.count < 2 and opt.ls_refresh == 1 or
				b.count < 3 and opt.ls_refresh == 2 or
				b.count < 4 and opt.ls_refresh == 3 or
				b.count < 5 and opt.ls_refresh == 4 or
				b.count < 6 and opt.ls_refresh == 5 or
				b.count < 7 and opt.ls_refresh == 6 or
				b.count < 8 and opt.ls_refresh == 7 or
				b.count < 9 and opt.ls_refresh == 8 then
					n = 324
			end

	
		-- MT refresh
		if opt.use_magma_totem_low then
			local _, totemname, start, duration = GetTotemInfo(1)
				if not totemname or totemname == "" then
				n = 58734
			end
   
			if start then
				local timeleft = start + duration - GetTime()	
					if timeleft < 6 then
						n = 58734
					end
			end
		end
		-- /MT refresh

		if spells[61657].cd < th then
			n = 61657
		end
		
		if spells[60103].cd < th then
			n = 60103
		end
		
		if spells[17364].cd < th then
			n = 17364
		end

		--es skipping
		b = myDebuffs[md_fs]
			if 
			spells[8042].cd < th and opt.fs_refresh == 0 or
			spells[8042].cd < th and b.time_left > 0.25 and opt.fs_refresh == 1 or
			spells[8042].cd < th and b.time_left > 0.50 and opt.fs_refresh == 2 or
			spells[8042].cd < th and b.time_left > 0.75 and opt.fs_refresh == 3 or
			spells[8042].cd < th and b.time_left > 1.00 and opt.fs_refresh == 4 or
			spells[8042].cd < th and b.time_left > 1.25 and opt.fs_refresh == 5 or
			spells[8042].cd < th and b.time_left > 1.50 and opt.fs_refresh == 6 or
			spells[8042].cd < th and b.time_left > 1.75 and opt.fs_refresh == 7 or
			spells[8042].cd < th and b.time_left > 2.00 and opt.fs_refresh == 8 or
			spells[8042].cd < th and b.time_left > 2.25 and opt.fs_refresh == 9 or
			spells[8042].cd < th and b.time_left > 2.50 and opt.fs_refresh == 10 or
			spells[8042].cd < th and b.time_left > 2.75 and opt.fs_refresh == 11 or
			spells[8042].cd < th and b.time_left > 3.00 and opt.fs_refresh == 12 or
			spells[8042].cd < th and b.time_left > 3.25 and opt.fs_refresh == 13 or
			spells[8042].cd < th and b.time_left > 3.50 and opt.fs_refresh == 14 or
			spells[8042].cd < th and b.time_left > 3.75 and opt.fs_refresh == 15 or
			spells[8042].cd < th and b.time_left > 4.00 and opt.fs_refresh == 16 or
			spells[8042].cd < th and b.time_left > 4.25 and opt.fs_refresh == 17 or
			spells[8042].cd < th and b.time_left > 4.50 and opt.fs_refresh == 18 or
			spells[8042].cd < th and b.time_left > 4.75 and opt.fs_refresh == 19 or
			spells[8042].cd < th and b.time_left > 5.00 and opt.fs_refresh == 20 or
			spells[8042].cd < th and b.time_left > 5.25 and opt.fs_refresh == 21 or
			spells[8042].cd < th and b.time_left > 5.50 and opt.fs_refresh == 22 or
			spells[8042].cd < th and b.time_left > 5.75 and opt.fs_refresh == 23 then
				n = 8042
			end	
		-- /es skipping
			
		-- Magma Totem
		
		if opt.use_magma_totem_high then
			local _, totemname, start, duration = GetTotemInfo(1)
				if not totemname or totemname == "" then
				n = 58734
			end
   
			if start then
				local timeleft = start + duration - GetTime()	
					if timeleft < 2 then
						n = 58734
					end
			end
		end
		
		-- /Magma Totem
		
		b = myDebuffs[md_ss]
		if b.active == false and spells[17364].cd < th then
			n = 17364
		end
 
		b = myDebuffs[md_fs]
		if b.active == false and spells[49233].cd < th then
			n = 49233
		end
			
		if opt.use_shamanistic_rage_high then
			if spells[30823].cd < th then
				n = 30823
			end
		end
		
		if opt.use_shamanistic_rage_low then
			if 
				spells[30823].cd < th and (UnitPower("player") < 800) and opt.sr_use == 1 or
				spells[30823].cd < th and (UnitPower("player") < 1000) and opt.sr_use == 2 or
				spells[30823].cd < th and (UnitPower("player") < 1200) and opt.sr_use == 3 or
				spells[30823].cd < th and (UnitPower("player") < 1400) and opt.sr_use == 4 or
				spells[30823].cd < th and (UnitPower("player") < 1600) and opt.sr_use == 5 or
				spells[30823].cd < th and (UnitPower("player") < 1800) and opt.sr_use == 6 or
				spells[30823].cd < th and (UnitPower("player") < 2000) and opt.sr_use == 7 or
				spells[30823].cd < th and (UnitPower("player") < 2200) and opt.sr_use == 8 then
					n = 30823
			end
		end
			
		b = buffs[b_ls]
		if b.active == false then
			n = 324
		end
				
		b = buffs[b_mw]
		if b.count == 5 then
			n = 403
		end
		
		if spells[30823].cd < th and (UnitPower("player") < 439) then
			n = 30823
		end
		
		return n
		
	end
end
Faceroller:RegisterModule("enhance me!", Init, NextHit, Options)